package org.jutils.collection;

import java.util.*;
/**
 * This class is a static class and intended as a collection of tool
 * methods for default types like arrays and Vectors, that may not
 * be extended.
 * @author: M.Winkler, doubleSlash GmbH
 */
public class CollectionHelper {



    /**
         * Appends the elements in <code>data</code> to the vector <code>destination</code>.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return java.util.Vector
         * @param data java.util.Vector
         * @param destination java.util.Vector
         */
        public static List append(List data, List destination) {
            List returnValue = destination;
            if (returnValue == null)
                returnValue = new Vector();
            if ((data != null) && (data.size() > 0)) {
                for (int index = 0; index < data.size(); index++) {
                    returnValue.add(data.get(index));
                }
            }
            return returnValue;
        }



    /**
         * Calculates the cut of two vectors, that is the set of elements that are
         * present in v1 AND v2
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return java.util.Vector The result. if <code>destination != null</code>, then <code>destination</code>.
         * @param v1 java.util.Vector The first vector to be used. May be null or empty.
         * @param v2 java.util.Vector The second vector to be used. May be null or empty.
         * @param destination java.collection.List The destination to which the cut elements will be copied. Will NOT be emptied before putting elements into it. If null, then a new List will be created.
         */
        public static List createCut(List v1, List v2, List destination) {
            List returnValue = destination;

            // insert v1 in result set
            if (returnValue == null)
                returnValue = new Vector(v1);
            else
                returnValue.addAll(v1);

            // only keep whats in v2
            returnValue.retainAll(v2);
            return returnValue;
        }



    /**
         * Tests the helper
         * @param args an array of command-line arguments. Do not have any impact on the test.
         */
        public static void main(java.lang.String[] args) {
            System.out.println("Testing CollectionHelper ...");
            int errorCount = 0;
            int elemCount = 10;

            // check toVector
            Object[] testArray = new Object[elemCount];
            for (int index = 0; index < elemCount; index++) {
                testArray[index] = new Long(index);
            }

            System.out.println("test toVector ...");
            List resultList = toList(testArray, null);
            if (resultList.size() != elemCount) {
                System.err.println("length of vector incorrect! (" + resultList.size() + " ahould be " + elemCount + ")");
                errorCount++;
            }
            for (int index = Math.min(resultList.size(), testArray.length) - 1; index > -1; index--) {
                if (resultList.get(index) != testArray[index]) {
                    System.err.println("references do not match at position " + index + " of resultList!");
                    errorCount++;
                }
            }

            // --------------------------------------------------------------
            System.out.println("test createVectorCut ...");
            Vector v1 = new Vector(resultList);
            Vector v2 = new Vector(resultList);
            resultList.clear();
            createCut(v1, v2, resultList);
            if (!(resultList.equals(v1) && resultList.equals(v2))) {
                System.err.println("contents of vectors do not match!");
                System.err.println("v1 : " + v1.toString());
                System.err.println("v2 : " + v2.toString());
                System.out.println("cut: " + resultList.toString());
                errorCount++;
            }

            // --------------------------------------------------------------
            System.out.println("test removeDoubles ...");
            append(v2, v1);
            removeDoubles(v1);
            if (!v1.equals(v2)) {
                System.err.println("contents of vector does not match!");
                System.err.println("v1 (original): " + v2.toString());
                System.err.println("v1 (modified) : " + v1.toString());
                errorCount++;
            }

            // --------------------------------------------------------------
            System.out.println("Errors detected : " + errorCount);
        }



    /**
         * Searches for doubles and removes them, so that all elements are present only once.
         * Only checks for references, and not for values.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @param vector Vector
         */
        public static void removeDoubles(List list) {
            removeDoubles(list, false);
        }



    /**
         * Searches for doubles and removes them, so that all elements are present only once.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @param list List
          * @param alsoCheckValue boolean True if contents should be checked using <code>equals()</code> instead of a quick but dirty reference check.
         */
        public static void removeDoubles(List list, boolean alsoCheckValue) {
            if ((list != null) && (list.size() > 1)) {
                Object[] array = list.toArray();
                list.clear();
                for (int index = 0; index < array.length; index++) {
                    Object original = array[index];
                    boolean isPresent = false;
                    int innerIndex = index + 1;
                    while (!isPresent && (innerIndex < array.length)) {
                        isPresent = array[innerIndex] == original;
                        if (!isPresent && alsoCheckValue)
                            isPresent = array[innerIndex].equals(original);
                        if (!isPresent)
                            innerIndex++;
                    }
                    if (!isPresent)
                        list.add(original);
                }
            }
        }



    /**
         * Converts the passed Enumeration into an object array.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return java.lang.Object[]
         * @param enumeration java.util.Enumeration
         */
    public static Object[] toArray(Enumeration enumeration) {
        List aList= toList(enumeration);
        Object[] returnValue= aList.toArray();
        return returnValue;
    }



    /**
         * Converts the passed Iterator into an object array.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return java.lang.Object[]
         * @param enumeration java.util.Iterator
         */
    public static Object[] toArray(Iterator iterator) {
        List aList= toList(iterator);
        Object[] returnValue= aList.toArray();
        return returnValue;
    }



    /**
         * Puts the passed eolements into the passed vector.
         * Does neither check elements for null nor clear the result Vector.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return java.util.Vector
         * @param array java.lang.Object[]
         * @param destination java.util.Vector
         */
        public static List toList(Object[] array, List destination) {
            List returnValue = destination;
            if (array != null) {
                if (returnValue == null)
                    returnValue = new Vector();
                if (array.length > 0) {
                    for (int index = 0; index < array.length; index++) {
                        returnValue.add(array[index]);
                    }
                }
            }
            return returnValue;
        }



    /**
         * puts all elements of the passed enumeration into a List, and returns it.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @param enumeration java.util.Enumeration
         * @see java.util.Enumeration
         */
    public static Vector toList(Enumeration enumeration) {
        Vector returnValue= new Vector();
        while (enumeration.hasMoreElements())
            returnValue.add(enumeration.nextElement());
        return returnValue;
    }



    /**
         * puts all elements of the passed iterator into a Vector, and returns it.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @param iterator java.util.Iterator
         * @see java.util.Iterator
         */
    public static Vector toList(Iterator iterator) {
        Vector returnValue= new Vector();
        while (iterator.hasNext())
            returnValue.add(iterator.next());
        return returnValue;
    }



    /**
             * @author Michael Winkler, doubleSlash GmbH
             *
             * @return java.lang.StringBuffer
             * @param data java.lang.Object[]
             * @param opener java.lang.String
             * @param separator java.lang.String
             * @param closer java.lang.String
             * @param destination java.lang.StringBuffer
             */
        public static StringBuffer toStringBuffer(Object[] data, String opener, String separator, String closer, StringBuffer destination) {
            StringBuffer returnValue = destination;
            if (returnValue == null)
                returnValue = new StringBuffer();
            if (opener != null)
                returnValue.append(opener);
            if ((data != null) && (data.length > 0)) {
                boolean alreadyAppendedSomething = false;
                for (int index = 0; index < data.length; index++) {
                    Object singleData = data[index];
                    if (alreadyAppendedSomething)
                        returnValue.append(separator);
                    returnValue.append(singleData);
                    alreadyAppendedSomething = true;
                }
            }
            if (closer != null)
                returnValue.append(closer);
            return returnValue;
        }



    /**
     * Creates a hash key that caches the hash key value of the passed object.<br>
     * This is mainly to increase a hashing speed when using Strings, which recalc
     * the hash key every time the hashCode method is called.
     * @author: M.Winkler, doubleSlash GmbH
     * @return java.lang.Object an object usable as faster, caching hash key
     * @param value java.lang.Object
     */
    public static Object hashKey(Object value) {
        class HashKey {
            Object value_;
            int hashCode_;
            public HashKey(Object key) {
                value_ = key;
                hashCode_ = key.hashCode();
                return;
            }
            public int hashCode() {
                return hashCode_;
            }
            public boolean equals(Object o) {
                return value_.equals(o);
            }
        }
        return new HashKey(value);
    }
}

