package org.jutils.collection;

/**
 * This class defines the methods that is needed for a <i>Observer</i> of an instance of the type
 * <tt>org.jutils.collection.ExtendedMap</tt>.<br>
 * This mostly is inspired from the <i>Observer</i> pattern as defined by then gang of four,
 * and exercised by the classes <tt>java.util.Observer</tt>
 * and <tt>java.util.Observable</tt>.<b>
 * The speciality of this case only is that it additionaly provides the ability to observe named values,
 * as provided by the interface <tt>java.util.Map</tt> interface.<p>
 * For a simple usage demo, see the main method of org.jutils.collection.ExtendedMap.
 * @see org.jutils.collection.ExtendedMap
 * @see java.util.Observer
 * @see java.util.Observable
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 */
public interface MapObserver {



/**
 * This method is called by an <tt>org.jutils.collection.ObservedMap</tt> if a specific key (and its according value) has been added to the map.
 * At the time of call, the value already is contained by the map,  so an iterator onto
 * the map will reflect this fact.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param observableMap org.jutils.collection.ExtendedMap The firing (and changed) map.
 * @param key java.lang.Object The added key. This key is validated by a validator and contained by the map.
 * @param value java.lang.Object The corresponding value. This key is validated by a validator and contained by the map.
 */
void keyAdded(ExtendedMap observableMap, Object key, Object value);





/**
 * This method is called by an <tt>org.jutils.collection.ObservedMap</tt> if a specific key (and its according value) has been removed from the map.
 * At the time of call, the value is no more contained by the map,  so an iterator onto
 * the map will reflect this fact.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param observableMap org.jutils.collection.ExtendedMap The firing (and changed) map.
 * @param key java.lang.Object The key that has been removed. This key is no more contained by the map (its value too).
 * @param value java.lang.Object The corresponding value, for convenience purposes.
 */
void keyRemoved(ExtendedMap observableMap, Object key, Object value);





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param observableMap org.jutils.collection.ExtendedMap
 */
void mapCleared(ExtendedMap observableMap);





/**
 * This method is called by an <tt>org.jutils.collection.ObservedMap</tt> if a specific value has been changed.
 * At the time of call, the old value is no longer part of the map, and the new value already is, so an iterator onto
 *  the map will reflect this fact.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param observableMap org.jutils.collection.ExtendedMap The firing ExtendedMap (which contains the changed value).
 * @param key java.lang.Object A valid key (checked by an eventually set key validator).
 * @param oldValue java.lang.Object The old value (no more valid and no more contained by the map).
 * @param newValue java.lang.Object The new value (valid and contained by the map).
 * @see org.jutils.collection.ObservedMap
 */
void valueChanged(
   ExtendedMap observableMap,
   Object key,
   Object oldValue,
   Object newValue);
}
