package org.jutils.collection;
import java.text.*;/**
* Provides the subtile mechanisms used to access a sorted list of long values.
* <p>------------------------------------------------------------------------------
* <br>Name:     SortedLongList
* <br>Project:   jutils
* <br>Copyright: jutils.org, see license file.
* <br>Creation date: (25.07.2000 15:54:10)
* <br>Version:   <pre>$Id: SortedLongList.java,v 1.24 2001/02/07 18:49:50 winkler Exp $</pre>
* <br>------------------------------------------------------------------------------
* <p>
* @see DynamicList
* @author: M.Winkler, doubleSlash GmbH
**/
import java.util.*;
import org.jutils.*;
import java.io.*;

public class SortedLongList implements List, DynamicList {
   public static final int INITIAL_CAPACITY = 15;
   /** The concrete data array. */
   private long[] data_ = null;
   /** Number of elements stored in data. */
   private int count_ = 0;
   /** Number of elements the capacity is raised when capacity is exceeded. */
   private int raise_ = -1;
   private String openListChar_ = CharConstants.EMPTY_STRING;
   private String listSeparatorChar_ = "" + org.jutils.CharConstants.SEMICOLON;
   private String closeListChar_ = CharConstants.EMPTY_STRING;
   // The mutex used to synchronize upon.
   private Object mutex_ = new Object();
   private boolean dirty_ = false;
   // caches the hashcode of the object. Is only computed when dirty_ = true.
   private int hashcode_ = 0;
   /** The implanted instance recycler of this class. */
   private static List recycler_ = null;
   private static int minSize_ = 100;
   private static int maxSize_ = Integer.MAX_VALUE - 1;
   private static int instancesCreratedCount_ = 0;
   private static int instancesRecycledCount_ = 0;
   private static int instancesFinalizedCount_ = 0;
   private static int instancesAliveCount_ = 0;

   public static final SortedLongList EMPTY_INSTANCE = new SortedLongList(0, 0);



   /**
     * SortedLongList constructor comment.
     */
   public SortedLongList() {
     super();
     data_= new long[INITIAL_CAPACITY];
     raise_= INITIAL_CAPACITY;
     instancesCreratedCount_++;
     instancesAliveCount_++;
   }



   /**
     * SortedLongList constructor comment.
     */
   public SortedLongList(int initialSize) {
     super();
     data_= new long[initialSize];
     raise_= initialSize;
     instancesCreratedCount_++;
     instancesAliveCount_++;
   }



   /**
     * SortedLongList constructor comment.
     */
   public SortedLongList(int initialSize, int initialRaise) {
     super();
     data_= new long[initialSize];
     raise_= initialRaise;
     instancesCreratedCount_++;
     instancesAliveCount_++;
   }



   /**
   * Creaters a new SortedLongList out of a tokenizable String.
   * @see java.util.StringTokenizer
   */
   public SortedLongList(
     String stringRepresentation,
     String opener,
     String separator,
     String closer)
     throws ParseException {
     this();
     if (stringRepresentation != null) {
       stringRepresentation.trim();

       // remove opener string, if present.
       if (opener != null) {
         int startIndex= stringRepresentation.indexOf(opener);
         if (startIndex == 0)
            stringRepresentation= stringRepresentation.substring(opener.length());
       }

       // remove closer string, if present.
       if (closer != null) {
         int endIndex= stringRepresentation.lastIndexOf(closer);
         if (endIndex > -1)
            stringRepresentation= stringRepresentation.substring(0, endIndex);
       }

       // parse
       StringTokenizer tokenizer= new StringTokenizer(stringRepresentation, separator);
       int tokenCount= 0;
       while (tokenizer.hasMoreTokens()) {
         String token= tokenizer.nextToken();
         tokenCount++;
         try {
            long parsedLong= Long.parseLong(token);
            append(parsedLong);
         }
         catch (NumberFormatException nfe) {
            String message=
              "Could not parse token #" + tokenCount + ", which is '" + token + "'.";
            throw new ParseException(message, tokenCount);
         }
       }
     }
     // transfer parameters to attributes.
     openListChar_= opener;
     closeListChar_= closer;
     listSeparatorChar_= separator;

     // update recycler stats
     instancesCreratedCount_++;
     instancesAliveCount_++;
   }



   /**
   * Creates a new SortedLongList from out of a list of Numbers.
   * All elements in this list MUST be of type Number in order to function properly.
   * @exception ClassCastException If one of the elements in the list is not casteable to a Number.
   * @see java.lang.Number
   * @author: M.Winkler, doubleSlash NetBusiness GmbH
   *
   * @param list java.util.List
   */
   public SortedLongList(List list) {

     // determine initial size
     int initialSize= list != null ? list.size() : INITIAL_CAPACITY;

     // init data
     data_= new long[initialSize];
     raise_= initialSize;

     // transfer list entries to data

     if ((list != null) && (list.size() > 0)) {
       count_= list.size();
       for (int listIndex= 0; listIndex < count_; listIndex++) {
         Number singleNumber= (Number) list.get(listIndex);
         data_[listIndex]= singleNumber.longValue();
       }
       dirty_= true;
     }
     instancesCreratedCount_++;
     instancesAliveCount_++;
   }



   /**
   * Creates a new Sorted Long list that copies the state of the passed one.
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @param template org.jutils.collection.SortedLongList
   */
   public SortedLongList(SortedLongList template) {
     if (template != null) {
       synchronized (template.mutex_) {
         data_= new long[template.data_.length];
         System.arraycopy(template.data_, 0, data_, 0, template.count_);
         count_= template.count_;
         dirty_= template.dirty_;
         openListChar_= template.openListChar_;
         listSeparatorChar_= template.listSeparatorChar_;
         closeListChar_= template.closeListChar_;
       }
     }
     else {
       data_= new long[INITIAL_CAPACITY];
       raise_= INITIAL_CAPACITY;
     }
     instancesCreratedCount_++;
     instancesAliveCount_++;
   }



   /**
   * add method comment.
   */
   public void add(int index, java.lang.Object element) {
     throw new UnsupportedOperationException("Since this SortedLongList is sorted, you cannot specify a position to which an element should be added.");
   }



   /**
   * Adds the passed element to the list, if its an instance of Number; otherwise throws a ClassCastException.
   */
   public boolean add(java.lang.Object o) {
     boolean returnValue = false;
     Number n = (Number) o;
     append(n.longValue());
     returnValue = true;
     return returnValue;
   }



   /**
   * addAll method comment.
   */
   public boolean addAll(int index, java.util.Collection c) {
     throw new UnsupportedOperationException("Since this SortedLongList is sorted, you cannot specify a position to which an element should be added.");
   }



   /**
   * Adds all passed elements to the list, if they are an instance of Number; otherwise throws a ClassCastException.
   */
   public boolean addAll(java.util.Collection c) {
     boolean returnValue = false;
     if ((c != null) && (c.size() > 0)) {
       Iterator i = c.iterator();
       while (i.hasNext()) {
         Number n = (Number) i.next();
         append(n.longValue());
         returnValue = true;
       }
     }
     return returnValue;
   }



   /**
   * Adds a new value to the tail of the row. Does not sort the row until a reading access is done.
     * <br>Creation date: (25.07.2000 16:50:45)<br>
     * @author: M.Winkler, doubleSlash GmbH
     *
     * @param value long A new value.
     * @return SortedLongList this instance, to enable chaining.
     */
   public SortedLongList append(long value) {
     synchronized (mutex_) {
       if (data_.length == count_)
         pEnsureCapacity(count_ + raise_);
       data_[count_]= value;
       count_++;
       dirty_= true;
     }
     return this;
   }



   /**
   * Appends all values of the passed list into the local list, in a "quick and dirty" manner.
   * <br>Creation date: (25.07.2000 16:50:45)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @param values SortedLongList
   * @return SortedLongList this instance, to enable chaining.
   */
   public SortedLongList append(SortedLongList values) {
     if ((values != null) && (values.count() > 0)) {
       append(values, 0, Integer.MAX_VALUE);

     }
     return this;
   }



   /**
   * Appends the values of the passed list into the local list, in a "quick and dirty" manner.
   * <br>Creation date: (25.07.2000 16:50:45)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @param values SortedLongList
   * @param fromIndex int The first index to be copied.
   * @param toIndex int The last index to be copied.
   * @return SortedLongList this instance, to enable chaining.
   */
   public SortedLongList append(
     SortedLongList values,
     int fromIndex,
     int toIndex) {
     if (fromIndex < 0)
       throw new IllegalArgumentException("fromIndex < 0!");
     if (toIndex < fromIndex)
       throw new IllegalArgumentException("toIndex < fromIndex!");
     toIndex= Math.min(values.size() - 1, toIndex);

     if ((values != null) && (values.count_ > 0)) {
       synchronized (values.mutex_) {
         pEnsureCapacity(values.count_ + count_);
         synchronized (mutex_) {
            int copyCount= toIndex - fromIndex + 1;
            System.arraycopy(values.data_, fromIndex, data_, count_, copyCount);
            count_= count_ + copyCount;
            dirty_= true;
         }
       }
     }
     return this;

   }



   /**
     * capacity method comment.
     */
   public int capacity() {
     int returnValue = 0;
     synchronized (mutex_) {
       returnValue = data_.length;
     }
     return returnValue;
   }



   /**
   * Removes all elements from the list.
   */
   public void clear() {
     synchronized (mutex_) {
       count_ = 0;
       dirty_ = false;
     }
   }



   /**
   * Clones this list into a new instance.
   */
   public DynamicList cloneList() {
     SortedLongList returnValue= null;
     returnValue= newInstance();
     returnValue.append(this);
     returnValue.setSettingsLike(this);
     return returnValue;
   }



   /**
   * Returns true if the passed value is contained by the SortedLongList.
   * <br>Creation date: (26.07.2000 17:57:22)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @return boolean
   * @param value long
   */
   public boolean contains(long value) {
     boolean returnValue = false;
     synchronized (mutex_) {
       int index = pGetIndex(value);
       returnValue = (index < count_) && (data_[index] == value);
     }
     return returnValue;
   }



   /**
   * contains method comment.
   */
   public boolean contains(java.lang.Object o) {
     boolean returnValue = false;
     try {
       Number n = (Number) o;
       returnValue = contains(n.longValue());
     }
     catch (ClassCastException cce) {
       returnValue = false;
     }
     return returnValue;
   }



   /**
   * containsAll method comment.
   */
   public boolean containsAll(java.util.Collection c) {
     boolean returnValue = true;
     Iterator i = c.iterator();
     while (returnValue && i.hasNext()) {
       returnValue = contains(i.next());
     }
     return returnValue;
   }



   /**
     * <br>Creation date: (10.10.2000 10:00:00)<br>
     * @author: K.Krafft, doubleSlash GmbH
     *
     * @return int
     */
   public int count() {
     return count_;
   }



   /**
   * Creates a cut of both lists.
   * <br>Creation date: (27.07.2000 14:15:38)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @return org.jutils.collection.SortedLongList The result.
   * @param anotherList org.jutils.collection.SortedLongList
   * @param lsitToPutResultInto org.jutils.collection.SortedLongList The result destination. If null, a new SortedLongList will be generated.
   */
   public SortedLongList createCut(
     SortedLongList anotherList,
     SortedLongList listToPutResultInto) {
     SortedLongList returnValue= listToPutResultInto;
     if (returnValue == null) {
       returnValue= newInstance();
       returnValue.setSettingsLike(this);
       returnValue.clear();
     }

     // check for useless parameters.
     if ((anotherList == null) || (anotherList.count_ == 0) || (count_ == 0)) {
       return returnValue;
     }

     // do the job
     synchronized (mutex_) {
       synchronized (anotherList.mutex_) {

         // ensure consistency
         pReensureConsistency();

         // get data and the amount of data into some locals (for better source understanding)
         long[] data1= data_;
         int count1= count_;
         long[] data2= anotherList.data_;
         int count2= anotherList.count_;

         // get the start indices to start from
         int index1= pGetIndex(data2[0]);
         int index2= anotherList.pGetIndex(data1[0]);

         // get the indices to stop at.
         //         int list1StopIndex =
         int stopIndex1= Math.min(pGetIndex(data2[count2 - 1]), count1 - 1);
         int stopIndex2= Math.min(anotherList.pGetIndex(data1[count1 - 1]), count2 - 1);
         while ((index1 <= stopIndex1) && (index2 <= stopIndex2)) {
            long value1= data1[index1];
            long value2= data2[index2];
            if (value1 == value2) {
              returnValue.append(value1);
              index1++;
              index2++;
            }
            else {
              if (value1 < value2)
                index1++;
              else {
                index2++;
              }
            }
         }
       }
     }
     // Thats it!
     return returnValue;
   }



   /**
     * ensureCapacity method comment.
     */
   public int ensureCapacity(int newCapacity) {
     int returnValue = 0;
     synchronized (mutex_) {
       pReensureConsistency();
       pEnsureCapacity(newCapacity);
       returnValue = data_.length;
     }
     return returnValue;
   }



   /**
     * Compares two objects for equality. Returns a boolean that indicates
     * whether this object is equivalent to the specified object. This method
     * is used when an object is stored in a hashtable.
     * @param obj the Object to compare with
     * @return true if these Objects are equal; false otherwise.
     * @see java.util.Hashtable
     */
   public boolean equals(Object obj) {
     boolean returnValue = false;
     if (obj != null) {
       if (obj.getClass().equals(this.getClass())) {
         SortedLongList list2 = (SortedLongList) obj;
         // synchronize both lists
         synchronized (mutex_) {
            synchronized (list2.mutex_) {

              // since both lists are synchronized, now not-so-beautiful private
              // accesses are done. this is more fail safe than calling methods.
              if (count_ == list2.count_) {
                returnValue = true;
                for (int index = 0; returnValue && (index < count_); index++) {
                  returnValue = data_[index] == list2.data_[index];
                }
              }
            }
         }
       }
     }
     return returnValue;
   }



/**
 * get method comment.
 */
public java.lang.Object get(int index) {
   return org.jutils.runtime.Recycler.getLong(getLongAt(index));
}



   /**
     * getIndexOutOfBoundsBehavior method comment.
     */
   public int getIndexOutOfBoundsBehavior() {
     return THROW_EXCEPTION;
   }



   /**
     * <br>Creation date: (25.07.2000 16:49:23)<br>
     * @author: M.Winkler, doubleSlash GmbH
     *
     * @return long
     * @param index int
     */
   public long getLongAt(int index) {
     long returnValue = Long.MIN_VALUE;
     synchronized (mutex_) {
       pReensureConsistency();
       if ((index < 0) || (index >= count_))
         throw new IndexOutOfBoundsException("Index is " + index);
       returnValue = data_[index];
     }
     return returnValue;
   }



   /**
   * Returns the maximum value of this list.
   * <br>Creation date: (27.07.2000 14:06:05)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @return long
   */
   public long getMax() {
     if (count_ < 1)
       throw new IllegalStateException("No elements in list, therefore I cannot say whats the maximum value.");
     long returnValue = Long.MIN_VALUE;
     synchronized (mutex_) {
       if (dirty_) {
         for (int index = 0; index < data_.length; index++) {
            returnValue = Math.max(returnValue, data_[index]);
         }
       }
       else
         returnValue = data_[count_ - 1];
     }
     return returnValue;
   }



   /**
   * Returns the minimal value of this list.
   * <br>Creation date: (27.07.2000 14:06:05)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @return long
   */
   public long getMin() {
     if (count_ < 1)
       throw new IllegalStateException("No elements in list, therefore I cannot say whats the minimum value.");
     long returnValue = Long.MAX_VALUE;
     synchronized (mutex_) {
       if (dirty_) {
         for (int index = 0; index < data_.length; index++) {
            returnValue = Math.min(returnValue, data_[index]);
         }
       }
       else
         returnValue = data_[0];
     }
     return returnValue;
   }



   /**
     * getObjectAt method comment.
     */
   public Object getObjectAt(int index) {
     Object returnValue = null;
     synchronized (mutex_) {
       pReensureConsistency();
       if ((index < 0) || (index >= count_))
         throw new IndexOutOfBoundsException("Index is " + index);
       returnValue = new Long(data_[index]);
     }
     return returnValue;
   }



   /**
   * Insert the method's description here.
   * Creation date: (03.11.2000 09:56:58)
   * @return int
   */
   public int getRaise() {
     return raise_;
   }



   /**
     * Generates a hash code for the receiver.
     * This method is supported primarily for
     * hash tables, such as those provided in java.util.
     * @return an integer hash code for the receiver
     * @see java.util.Hashtable
     */
   public int hashCode() {
     synchronized (mutex_) {
       if (dirty_) {
         pReensureConsistency();
         hashcode_ = 0;
         for (int index = 0; index < count_; index++) {
            hashcode_ = hashcode_ + (int) (index * data_[index]);
         }
       }
     }
     return hashcode_;
   }



   /**
   * indexOf method comment.
   */
   public int indexOf(java.lang.Object o) {
     int returnValue = -1;
     Number n = (Number) o;
     synchronized (mutex_) {
       int index = pGetIndex(n.longValue());
       if (index < count_) {
         returnValue = index;
       }
     }
     return returnValue;
   }



   /**
     * isAutoResizing method comment.
     */
   public boolean isAutoResizing() {
     return true;
   }



   /**
   * isEmpty method comment.
   */
   public boolean isEmpty() {
     return count_ < 1;
   }



   /**
   * iterator method comment.
   */
   public java.util.Iterator iterator() {
     Object[] array = toArray();
     Iterator returnValue = new GenericIterator(array);
     return returnValue;
   }



   /**
   * lastIndexOf method comment.
   */
   public int lastIndexOf(java.lang.Object o) {
     int returnValue = -1;
     returnValue = indexOf(o);
     return returnValue;
   }



   /**
   * listIterator method comment.
   */
   public java.util.ListIterator listIterator() {
     throw new UnsupportedOperationException("listIterator() not supported by SortedLongList.");
   }



   /**
   * listIterator method comment.
   */
   public java.util.ListIterator listIterator(int index) {
     throw new UnsupportedOperationException("listIterator(int) not supported by SortedLongList.");
   }



   /**
   * Lists the stats of the internal instance recycler.
   * Creation date: (02.11.2000 17:10:39)
   * @param printWriter java.io.PrintWriter
   */
   public static void listRecyclerStats(OutputStream outStream) {
     PrintWriter printWriter= new PrintWriter(outStream);
     printWriter.println("Statistics of internal Recycler of SortedLongList:");
     printWriter.println("" + instancesCreratedCount_ + " instances created.");
     printWriter.println("" + instancesRecycledCount_ + " instances recycled.");
     printWriter.println("" + instancesFinalizedCount_ + " instances finalized.");
     printWriter.println("" + instancesAliveCount_ + " instances alive.");

     printWriter.println(
       "" + (recycler_ != null ? recycler_.size() : 0) + " instances in recycler.");
     printWriter.flush();

   }



   /**
     * <br>Creation date: (25.07.2000 17:03:18)<br>
     * @author: M.Winkler, doubleSlash GmbH
     *
     * @param args java.lang.String[]
     */
   public static void main(String[] args) {

     // initializing
     int initialAmount= 50;
     int amount= initialAmount;
     int errorCount= 0;
     System.out.println("Testing SortedLongList...");
     System.out.println(
       "Putting "
         + amount
         + " elements each into two lists, one directly, the other via StringRepresentation-constructor.");
     StringBuffer stringRepresentationBuffer= new StringBuffer("{");
     SortedLongList list_2= SortedLongList.newInstance();
     list_2.ensureCapacity(amount);
     long currentTimeMillis= System.currentTimeMillis();
     for (int index= 0; index < amount; index++) {
       if (index > 0)
         stringRepresentationBuffer.append(";");
       stringRepresentationBuffer.append((int) (Math.random() * amount));
       list_2.append((int) (Math.random() * amount));
     }
     long timeUsed= System.currentTimeMillis() - currentTimeMillis;
     System.out.println("Time used: " + timeUsed + " ms for both.");
     System.out.println(
       "Creating the other list via StringRepresentation constructor ...");
     stringRepresentationBuffer.append("blabla");
     System.out.println(
       "String representation is " + stringRepresentationBuffer.toString());
     SortedLongList list_1= null;
     try {
       list_1=
         new SortedLongList(stringRepresentationBuffer.toString(), "{", ";", "blabla");
     }
     catch (ParseException pe) {
       errorCount++;
       System.err.println("Could not generate second list! Ending test!");
       System.exit(1);
     }

     // sort list
     System.out.println("Sort lists by accessing 0th element...");
     currentTimeMillis= System.currentTimeMillis();
     list_1.getLongAt(0);
     list_2.getLongAt(0);
     timeUsed= System.currentTimeMillis() - currentTimeMillis;
     System.out.println("Time used: " + timeUsed + " ms for both, that is " + (int)
       (timeUsed / 2) + " for each.");
     amount= list_1.size();
     System.out.println("Elements remaining in list_1: " + list_1.size());
     System.out.println("Elements remaining in list_2: " + list_2.size());

     // second access
     System.out.println("Second access...");
     currentTimeMillis= System.currentTimeMillis();
     list_1.getLongAt(0);
     timeUsed= System.currentTimeMillis() - currentTimeMillis;
     System.out.println("Time used: " + timeUsed + " ms");

     // do automatic semantic test
     // ---------------------------------------------------------------
     System.out.println("Checking sort order and contains()...");
     long previousValue= list_1.getLongAt(0);
     currentTimeMillis= System.currentTimeMillis();
     for (int index= 1; index < amount; index++) {
       long actualValue= list_1.getLongAt(index);
       if (actualValue <= previousValue) {
         System.err.println(
            "Element #"
              + index
              + "(="
              + actualValue
              + ") <= previous("
              + previousValue
              + ") !");
         errorCount++;
       }
       if (!list_1.contains(actualValue)) {
         errorCount++;
         System.err.println(
            "Element #" + index + "(=" + actualValue + ") has not been found!");
       }
       previousValue= actualValue;
     }
     timeUsed= System.currentTimeMillis() - currentTimeMillis;
     System.out.println("Time used: " + timeUsed + " ms");

     // check for impossible elements
     if (list_1.contains(-1)) {
       System.err.println("Checking for non existing element did return true!");
       errorCount++;
     }

     // check createCut
     System.out.println("Checking createCut()...");
     currentTimeMillis= System.currentTimeMillis();
     SortedLongList cut= list_1.createCut(list_2, null);
     timeUsed= System.currentTimeMillis() - currentTimeMillis;
     System.out.println(
       "Time used to create cut :"
         + timeUsed
         + " ms. Cut contains "
         + cut.size()
         + " elements.");

     // checking cut contents if they are present in BOTH lists
     System.out.println("Checking cut contents ...");
     for (int index= 0; index < cut.size(); index++) {
       long value= cut.getLongAt(index);
       if (!(list_1.contains(value) && list_2.contains(value))) {
         System.err.println(
            "Cut contains an element thta is not contained of BOTH other lists, at index "
              + index
              + "/"
              + (cut.size() - 1));
         errorCount++;
       }
     }

     // checking wether all elements present in list_1 AND list_2 are also present in cut
     System.out.println(
       "checking wether all elements present in list_1 AND list_2 are also present in cut. ...");
     for (int index= 0; index < list_1.size(); index++) {
       long value= list_1.getLongAt(index);
       if (list_2.contains(value) && !cut.contains(value)) {
         System.err.println(
            "Cut misses a common element of list_1 and list_2 at index "
              + index
              + "/"
              + (list_1.size() - 1)
              + " of list_1.:"
              + value);
         errorCount++;
       }
     }
     // same check, other direction
     for (int index= 0; index < list_2.size(); index++) {
       long value= list_2.getLongAt(index);
       if (list_1.contains(value) && !cut.contains(value)) {
         System.err.println(
            "Cut misses a common element of list_2 and list_1 at index "
              + index
              + "/"
              + (list_2.size() - 1)
              + " of list_2 :"
              + value);
         errorCount++;
       }
     }

     // print lists
     /*
     System.out.println("list_1 :" + list_1.toString());
     System.out.println("list_2 :" + list_2.toString());
     System.out.println("cut    :" + cut.toString());
     */

     // Check low content management
     System.out.println("checking wether a single elements will be found. ...");
     list_1.clear();
     list_1.append(5);
     if (!list_1.contains(5)) {
       System.out.println("Single element not detected.");
       errorCount++;
     }
     System.out.println("checking wether one of two elements will be found. ...");
     list_1.append(6);
     if (!list_1.contains(5)) {
       System.out.println("Lower element not detected.");
       errorCount++;
     }
     if (!list_1.contains(6)) {
       System.out.println("Higher element not detected.");
       errorCount++;
     }
     System.out.println("checking negative elements  ...");
     list_1.append(-100);
     if (!list_1.contains(-100)) {
       System.out.println("negative element not detected.");
       errorCount++;
     }
     if (list_1.getMin() != -100) {
       System.out.println("negative element not recognized as minimum.");
     }

     // check the toArray-methods
     System.out.println("Checking toArray() ...");
     Object[] array= list_1.toArray();
     try {
       Number n1= (Number) array[0];
       if (n1.longValue() != -100) {
         System.err.println("Wronmg value in Number 1 :" + n1);
         errorCount++;
       }
       else
         System.out.println("Number 1 checks.");
       Number n2= (Number) array[1];
       if (n2.longValue() != 5) {
         System.err.println("Wronmg value in Number 2 :" + n2);
         errorCount++;
       }
       else
         System.out.println("Number 2 checks.");
       Number n3= (Number) array[2];
       if (n3.longValue() != 6) {
         System.err.println("Wronmg value in Number 3 :" + n3);
         errorCount++;
       }
       else
         System.out.println("Number 3 checks.");
     }
     catch (ClassCastException cce) {
       System.err.println("Cannot cast array results to number.");
       errorCount++;
     }

     // check List constructor
     System.out.println("Checking List constructor ...");
     // first, convert to a list
     List anyList= CollectionHelper.toList(list_2.toArray(), null);
     // then, reparse to a SortedLongList
     SortedLongList generatedList= new SortedLongList(anyList);
     // check contents of the two lists
     cut= generatedList.createCut(list_2, null);
     // sizes have to match in order to determine a correct function.
     if (cut.size() != list_2.size()) {
       System.out.println(
         "Cut size does mismatch! Is "
            + cut.size()
            + ", but should be "
            + list_2.size()
            + "!");
       errorCount++;
     }

     // checking partial append
     System.out.println("Checking partial append ...");

     list_1.clear();
     for (int index= 1; index <= 5; index++) {
       list_1.append(index);
     }
     list_2.clear();
     for (int index= 6; index <= 10; index++) {
       list_2.append(index);
     }

     list_1.append(list_2, 1, 3);
     if (list_1.size() != 8) {
       System.out.println("Size does not match!");
       errorCount++;
     }
     else {
       cut= list_1.createCut(list_2, null);
       if (cut.size() != 3) {
         System.out.println("Size of cut does not match!");
         errorCount++;

       }
     }

     // checking remove (SortedLongList)
     System.out.println("Checking remove(SortedLongList) ...");
     list_1.clear();
     list_1.append(1).append(2).append(3);
     list_2.clear();
     list_2.append(3).append(4).append(5);
     list_1.remove(list_2);
     if (list_1.size() != 2) {
       System.out.println("Size does not match! Should be 2, but is " + list_1.size());
       errorCount++;
     }

     // listing stats
     SortedLongList.listRecyclerStats(System.out);

     // print result
     // ---------------------------------------------------------------
     System.out.println();
     System.out.println(String.valueOf(errorCount) + " errors detected. ");
   }



   /**
   * Returns a new instance. If any in the recycler, then return this one. Otherwise, create one.
   * Creation date: (02.11.2000 16:39:52)
   * @return org.jutils.collection.SortedLongList
   */
   public static SortedLongList newInstance() {
     SortedLongList returnValue= null;

     // do we have something in the recycler?
     if ((recycler_ == null) || (recycler_.size() < 1)) {
       returnValue= new SortedLongList();
     }
     else {

       // as we have something in the recycler, we take this one.
       synchronized (recycler_) {
         int lastPosition= recycler_.size() - 1;
         returnValue= (SortedLongList) recycler_.remove(lastPosition);
         instancesRecycledCount_++;
       }

     }
     return returnValue;
   }



   /**
   * Returns a new instance. If any in the recycler, then return this one. Otherwise, create one.
   * Creation date: (02.11.2000 16:39:52)
   * @param minCapacity The minimum capacity the returned list has to have.
   * @param raise int The raise the returned list should implement.
   * @return org.jutils.collection.SortedLongList
   */
   public static SortedLongList newInstance(int minCapacity, int raise) {
     SortedLongList returnValue= null;

     // return a default instance from the recycler.
     returnValue= newInstance();
     returnValue.setRaise(raise);
     returnValue.ensureCapacity(minCapacity);

     return returnValue;
   }



   /**
   * Returns a new instance. If any in the recycler, then return this one. Otherwise, create one.
   * Creation date: (02.11.2000 16:39:52)
   * @param template SortedLongList A template instance. Use this to copy all settings and contents from the template instance to the new instance.
   * @return org.jutils.collection.SortedLongList
   */
   public static SortedLongList newInstance(SortedLongList template) {
     SortedLongList returnValue= null;

     // call the other method
     returnValue= newInstance();
     if (template != null) {
       returnValue.setSettingsLike(template);
       returnValue.append(template);
     }
     return returnValue;
   }



   /**
   * Puts this instance into the recycler instead of dropping it into the gc.
   * Creation date: (02.11.2000 16:37:56)
   */
   public void recycle() {
     this.clear();
     if (recycler_ == null)
       recycler_= new Vector(minSize_);
     if (recycler_.size() < maxSize_) {
       recycler_.add(this);
     }
   }



   public java.lang.Object remove(int index) {
     if ((index < 0) || (index >= count_))
       throw new IndexOutOfBoundsException("Invalid index :" + index);
     long value = data_[index];
     removeLong(value);
     return new Long(value);
   }



   /**
   * remove method comment.
   */
   public boolean remove(java.lang.Object o) {
     boolean returnValue = false;
     Number n = (Number) o;
     returnValue = removeLong(n.longValue());
     return returnValue;
   }



   /**
   * Removes all elements of the passed list from the list.
   * Creation date: (02.11.2000 16:20:30)
   * @param sortedLongList org.jutils.collection.SortedLongList
   * @return boolean true if at least one element has been removed.
   */
   public boolean remove(SortedLongList sortedLongList) {
     boolean returnValue= false;
     if ((sortedLongList != null) && (sortedLongList.size() > 0) && (count_ > 0)) {

       synchronized (mutex_) {
         for (int index= sortedLongList.count_ - 1; index > -1; index--) {
            long value= sortedLongList.getLongAt(index);
            returnValue= returnValue || this.removeLong(value);
         }

       } // end of synchronized
     } // end of if
     return returnValue;
   }



   /**
   * removeAll method comment.
   */
   public boolean removeAll(java.util.Collection c) {
     return false;
   }



   /**
     * Removes this element from the list. Dirty implementation. May be optimizable.
     * <br>Creation date: (25.07.2000 16:53:03)<br>
     * @author: M.Winkler, doubleSlash GmbH
     *
     * @return boolean True if the element has been found, false otherwise.
     */
   public boolean removeLong(long value) {
     boolean returnValue = false;
     if (count_ > 0) {
       synchronized (mutex_) {

         // remove all occurrences
         for (int index = count_ - 1; index > -1; index--) {
            if (data_[index] == value) {
              returnValue = true;

              // if this is NOT the last element of the array,
              if (index < count_ - 1) {
                // copy the last elements one position to the forward.
                System.arraycopy(data_, index + 1, data_, index, count_ - index - 1);
              }
              count_--;
            }
         }
       }
     }
     return returnValue;
   }



   /**
   * retainAll method comment.
   */
   public boolean retainAll(java.util.Collection c) {
     return false;
   }



   /**
   * set method comment.
   */
   public java.lang.Object set(int index, java.lang.Object element) {
     throw new UnsupportedOperationException("Since a sortedLongList does not explicicely retain positions, you cvannot define to where insert an object.");
   }



   /**
     * setListChars method comment.
     */
   public void setListChars(String openBracket, String separator, String closeBracket) {
     synchronized (mutex_) {
       openListChar_ = openBracket;
       listSeparatorChar_ = separator;
       closeListChar_ = closeBracket;
     }
   }



   /**
   * Insert the method's description here.
   * Creation date: (03.11.2000 09:56:58)
   * @param newRaise int
   */
   public void setRaise(int newRaise) {
     raise_= newRaise;
   }



   /**
   * Sets the bounds of the inplanted instance recycler.
   * Creation date: (02.11.2000 16:45:16)
   * @param minSize int the minimum capacity of the recycler.
   * @param maxSize int The maximum capacity in the recycler. If the actual size is higher, then the actual size is adapted.
   */
   public static void setRecyclerBounds(int minSize, int maxSize) {
     if ((minSize_ < 0) || (maxSize < minSize_)) {
       String message= "Illegal arguments : minsize=" + minSize + "maxSize=" + maxSize;
       throw new IllegalArgumentException(message);
     }

     minSize_= minSize;
     maxSize_= maxSize;

     // check if we have to adapt the size of the recycler.
     if ((recycler_ != null) && (recycler_.size() > maxSize_)) {
       // shrink recycler.

       synchronized (recycler_) {
         for (int index= recycler_.size() - 1; index > maxSize; index--) {
            // remove this element from the recycler
            recycler_.remove(index);
         }

       }
     }
   }



   /**
   * Sets the settings of this instance to be according to the settings of the other class.
   * The settings are : list opener, separator and closer, and capacity raise value.
   * Creation date: (02.11.2000 16:59:49)
   * @param sortedLongList org.jutils.collection.SortedLongList
   */
   public void setSettingsLike(SortedLongList sortedLongList) {
     if (sortedLongList == null)
       throw new IllegalArgumentException("Sorry, cant handle null references as arguments.");
     listSeparatorChar_= sortedLongList.listSeparatorChar_;
     openListChar_= sortedLongList.openListChar_;
     closeListChar_= sortedLongList.closeListChar_;
     raise_= sortedLongList.raise_;
     ensureCapacity(sortedLongList.capacity());

   }



   /**
   * size method comment.
   */
   public int size() {
     int returnValue = 0;
     synchronized (mutex_) {
       pReensureConsistency();
       returnValue = count_;
     }
     return returnValue;
   }



   /**
   * subList method comment.
   */
   public java.util.List subList(int fromIndex, int toIndex) {
     if (fromIndex > toIndex)
       throw new IllegalArgumentException("fromIndex > toIndex!");
     if (fromIndex < 0)
       throw new IndexOutOfBoundsException(
         "Illegal paramenters : start index < 0: " + fromIndex);

     SortedLongList returnValue= new SortedLongList(count_ + 5);
     synchronized (mutex_) {
       if (toIndex >= count_)
         toIndex= count_ - 1;
       pReensureConsistency();
       for (int index= fromIndex; index <= toIndex; index++) {
         returnValue.append(data_[index]);
       }
     }
     return returnValue;
   }



   /**
   * toArray method comment.
   */
   public java.lang.Object[] toArray() {
     return toArray(null);
   }



/**
 * toArray method comment.
 */
public java.lang.Object[] toArray(java.lang.Object[] a) {
   Object[] returnValue = a;
   synchronized (mutex_) {
     pReensureConsistency();
     if (returnValue == null)
       returnValue = new Object[count_];
     for (int index = 0; index < returnValue.length; index++) {
       returnValue[index] = new Long(data_[index]);
     }
   }
   return returnValue;
}



   /**
     * Returns a String that represents the value of this object.
     * @return a string representation of the receiver
     */
   public String toString() {
     return toString(openListChar_, listSeparatorChar_, closeListChar_);
   }



   /**
     * toString method comment.
     */
   public String toString(String openBracket, String separator, String closeBracket) {
     StringBuffer returnValue = new StringBuffer();
     synchronized (mutex_) {
       pReensureConsistency();
       if (openBracket != null)
         returnValue.append(openBracket);
       if (count_ > 0) {
         for (int index = 0; index < count_; index++) {
            if ((index > 0) && (separator != null))
              returnValue.append(separator);
            returnValue.append(data_[index]);
         }
       }
       if (closeBracket != null)
         returnValue.append(closeBracket);
     }
     return returnValue.toString();
   }



   /**
     * Code to perform when this object is garbage collected.
     *
     * Any exception thrown by a finalize method causes the finalization to
     * halt. But otherwise, it is ignored.
     */
   protected void finalize() throws Throwable {
     // Insert code to finalize the receiver here.
     // This implementation simply forwards the message to super.  You may replace or supplement this.
     super.finalize();
     mutex_= null;
     data_= null;
     count_= 0;
     instancesFinalizedCount_++;
     instancesAliveCount_--;
   }



   /**
     * ensureCapacity method comment.
     */
   private int pEnsureCapacity(int newCapacity) {
     if (newCapacity > data_.length) {
       long[] newData = new long[newCapacity];
       System.arraycopy(data_, 0, newData, 0, count_);
       data_ = newData;
     }
     return data_.length;
   }



   /**
   * Returns the index on which this value is found.
   * If not present, it will return the index on which it would be inserted.
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return int
   * @param value long
   */
   private int pGetIndex(long value) {
     int returnValue = -1;


     // shortcut
     if (count_ == 0)
       return 0;

     // that sucks, but it has to be...

     // check wether there only is one element in the data.
     if (count_ < 2) {
       if (value == data_[0])
         returnValue = 0;
       else
         returnValue = value <= data_[0] ? 0 : 1;
     }

     // there are more than 1 element in the data
     else {

       // do a binary search
       pReensureConsistency();
       boolean found = false;

       // initialize edge values
       int lowerIndex = 0;
       int higherIndex = count_ - 1;
       while (!found && (lowerIndex + 1 < higherIndex)) {

         // adjust margin
         int midIndex = (int) (higherIndex + lowerIndex) / 2;
         if (value > data_[midIndex])
            lowerIndex = midIndex;
         else
            higherIndex = midIndex;

         // check if the element has been found
         found = (value == data_[lowerIndex]) || (value == data_[higherIndex]);
       }
       returnValue = higherIndex + 1;
       if (value <= data_[higherIndex]) {
         returnValue = higherIndex;
       }
       if (value <= data_[lowerIndex])
         returnValue = lowerIndex;
     }
     return returnValue;
   }



   /**
     * <br>Creation date: (25.07.2000 16:10:31)<br>
     * @author: M.Winkler, doubleSlash GmbH
     *
     */
   private void pReensureConsistency() {
     if ((count_ > 1) && dirty_) {

       // sort array
       Arrays.sort(data_, 0, count_);

       // remove doubles
       long previousValue = data_[0];
       int destIndex = 0;
       for (int index = 1; index < count_; index++) {
         long actualLong = data_[index];
         if (previousValue != actualLong) {
            destIndex++;
            data_[destIndex] = actualLong;
         }
         previousValue = actualLong;
       }

       // adjust count_
       count_ = destIndex + 1;
     }
     dirty_ = false;
   }
}
