package org.jutils.jdbc;

import java.sql.*;
import java.io.*;
/**
 * Provides a fully automated connection pool.<br>
 * The pool is built by passing an url , a user, password and a pool size to a constructor.
 * After this, the pool catches an amount of <code>Connection</code>s from the
 * <code>DriverManager</code> and encapsulates them into <code>PooledConnections</code>.<br>
 * To get such a pooled connection, simply call <code>ConnectionPool.getConnection()</code>,
 * or <code>PooledConnection.getConnection()</code>. Second method simply passes the call to
 * the first method.<p>
 * To release such a <code>PooledConnection</code>, simply call close() (which does NOT close the capsulated
 * connection), or explicitely call <code>PooledConnection.repool()</code>. Both <code>repool()</code> and
 * <code>close()</code> put the instance back into the pool.<br>
 * To destroy the pool, call close(), which will start to collect all conections, close and
 * invalidate them, and afterwards invalidates the pool.<p>
 * <strong>WARNING!</strong><br>
 * DONNOT </code>destroy()</code> a <code>PooledConnection</code> by yourself! <p>
 * If a <code>ConnectionPool</code> is closed, then all buzy PooledConnections keep valid and
 * usable until they are <code>repool()</code>ed or <code>close()</code>d.<p>
 * ------------------------------------------------------------------------------	<br>
 * Name:     	ConnectionPool														<br>
 * Project:   	jutils																<br>
 * Copyright: 																		<br>
 * Comment:																			<br>
 * Creation date: (28.06.2000 18:09:03)  											<br>
 * Version:   <pre>$Id: ConnectionPool.java,v 1.8 2000/08/22 10:03:27 winkler Exp $</pre> 													<br>
 * ------------------------------------------------------------------------------	<br>
 * @author: M.Winkler, doubleSlash GmbH
 */
public class ConnectionPool {
    public static final int THREAD_WAIT_MILLIS = 500;
    private PooledConnection[] pool_;
    private int count_ = 0;
    private int waitCount_ = 0;
    private int loopCount_ = 0;
    private int initialCount_ = 0;
    private int instancesOutwardCount_ = 0;
    private int instancesOutwardMax_ = 0;
    private boolean isOpen_ = true;
    private String url_ = null;
    private String user_ = null;
    private String pass_ = null;
    private boolean debug_ = false;



/**
 * Does nothing. Simply overwrites the default constructor to prohibit use.
 */
private ConnectionPool() {
   super();
}



/**
 * Creates a new open Connection pool.
 * The pool size is a measure for the amount tof connections that is requested from the
 * DriverManager by getting the assigned number of Connections in forward.
 * Creation date: (28.06.2000 18:12:08)
 * @param: <|>
 * @return:
 * @author: M.Winkler, doubleSlash GmbH
 *
 * @param url java.lang.String
 * @param user java.lang.String
 * @param pass java.lang.String
 * @param count int
 * @exception java.sql.SQLException The exception description.
 */
public ConnectionPool(String url, String user, String pass, int count) throws SQLException {
    // allocating memory
    pool_ = new PooledConnection[count];
    url_ = url;
    user_ = user;
    pass_ = pass;

    // get this bunch of connections.
    for (int index = 0; index < count; index++) {

        // get raw connection from driver
        Connection c = pGetNativeConnection();

        // create wrapper
        PooledConnection wrapper = new PooledConnection(c, this);
        wrapper.lock();

        // store wrapper
        pool_[index] = wrapper;
    }
    count_ = count;
    initialCount_ = count;
}



/**
 * Marks a pool to be removed from memory. If done so, all locally stored instances will be destroyed.
 * No notification of buzy connections will be done, so this connections will be destroyed when repooled.
 * Creation date: (29.06.2000 11:37:47)
 * @param: <|>
 * @return:
 * @author: M.Winkler, doubleSlash GmbH
 *
 */
public void close() {
    if (!isOpen_) {
        throw new IllegalStateException("ConectionPool already closed!");
    }
    if (debug_) {
        System.out.println("ConnectionPool : closing ...");
    }
    // block use from now on
    isOpen_ = false;

    // remove and destroy pooled connections
    synchronized (pool_) {
        // is there something in the pool?!?
        if (count_ > 0)
            for (int index = 0; index < count_; index++) {
                // destroy connection in pool
                pool_[index].destroy();

                // remove reference
                pool_[index] = null;
            }
        count_ = 0;
    }
    if (debug_) {
        System.out.println("ConnectionPool : closed.");
        printStatistics(System.out);
    }
    if (instancesOutwardCount_ < 1)
        destroy();
}



/**
 * @author Michael Winkler, doubleSlash GmbH
 * 
 */
private void destroy() {
    if (isDebug()) {
        System.out.println("ConnectionPool.destroy() : nullifying references ...");
    }
    pool_ = null;
    count_ = 0;
    instancesOutwardCount_ = 0;
    url_ = null;
    user_ = null;
    pass_ = null;
    if (isDebug()) {
        System.out.println("ConnectionPool.destroy() : done.");
    }
}



/**
 * If this happenes, then some requirements are met:
 * <li>all <code>PooledConnections</code> handled by this instance are returned via <code>takeInstance()</code>.</li>
 * <li>there is no more reference to this pool (except the finalizer's).</li>
 * Creation date: (29.06.2000 11:36:14)
 * @param: <|>
 * @return:
 * @author: M.Winkler, doubleSlash GmbH
 *
 * @exception java.lang.Throwable The exception description.
 */
protected void finalize() throws java.lang.Throwable {
    if (instancesOutwardCount_ > 0)
        throw new SQLException("Cannot finalize pool; still connections out!");
    pool_ = null;
    if (isDebug())
        System.out.println("PooledConnection : finalized.");
}



/**
 * Gets a connection from the pool. If there is no connection in the pool, the method waits until a connection is in the pool.
 * The found connection is deleted from the array (its index position is set null).
 * If this pool already is closed, then an IllegalStateException is thrown.
 * Creation date: (28.06.2000 19:16:37)
 * @return org.jutils.jdbc.PooledConnection
 * @author: M.Winkler, doubleSlash GmbH
 */
public PooledConnection getConnection() {

    // check wether the pool is opened
    if (!isOpen_)
        throw new IllegalStateException("Pool is closed!");

    // initialize
    PooledConnection returnValue = null;
    do {

        // is something in the pool?!?
        synchronized (pool_) {
            if (count_ > 0) {
                // yes, so take this one out of the pool
                count_--;
                instancesOutwardCount_++;
                if (instancesOutwardCount_ > instancesOutwardMax_)
                    instancesOutwardMax_ = instancesOutwardCount_;
                returnValue = pool_[count_];
                pool_[count_] = null;
                if (isDebug()) {
                    System.out.println("ConnectionPool : Gave out another connection.");
                    printStatistics(System.out);
                }
            }
        }
        if (returnValue == null) {
            // wait a little while
            waitCount_++;
            if (instancesOutwardCount_ > 0) {
                try {
                    if (isDebug()) {
                        System.out.println("ConnectionPool : Waiting for connection.");
                        printStatistics(System.out);
                    }
                    Thread.currentThread().wait(THREAD_WAIT_MILLIS);
                }
                catch (InterruptedException ie) {
                    System.err.println("Interrupted?!?");
                    ie.printStackTrace(System.err);
                }
                waitCount_--;
            }
            if (returnValue == null) {
                loopCount_++;
                if (debug_)
                    System.out.println("ConnectionPool.getConnection() relooping.");
            }
            else
                throw new IllegalStateException("cannot give out another connection : too few connections !");
        }
    }
    while (returnValue == null); // as long as nothing is found

    // unlock instance
    returnValue.unlock();

    // give it back
    return returnValue;
}



/**
 * Creation date: (29.06.2000 15:27:41)
 * @param: <|>
 * @return:
 * @author: M.Winkler, doubleSlash GmbH
 *
 * @return java.lang.String
 */
public String getJdbcUrl() {
   return url_;
}



/**
 * Returns true if this ConnectionPool is set to run in debug mode.
 * @author Michael Winkler, doubleSlash GmbH
 * 
 * @return boolean
 */
public boolean isDebug() {
    return debug_;
}



/**
 * Creation date: (29.06.2000 11:42:55)
 * @param: <|>
 * @return:
 * @author: M.Winkler, doubleSlash GmbH
 *
 * @return boolean
 */
public boolean isOpen() {
   return isOpen_;
}



/**
 * @author Michael Winkler, doubleSlash GmbH
 * 
 * @return java.sql.Connection
 */
private Connection pGetNativeConnection() throws SQLException {
    Connection c = null;
    try {
        c = DriverManager.getConnection(url_, user_, pass_);
    }
    catch (SQLException sqle) {
        System.err.println("ConnectionPool.pGetNativeConnection() : Cannot get connection from DriverManager : " + sqle.toString());
        throw sqle;
    }
    return c;
}



/**
 * @author Michael Winkler, doubleSlash GmbH
 * 
 * @param printStream java.io.PrintStream
 */
public void printStatistics(PrintStream printStream) {
    PrintWriter w = new PrintWriter(printStream);
    printStatistics(w);
}



/**
 * @author Michael Winkler, doubleSlash GmbH
 * 
 * @param printWriter java.io.PrintWriter
 */
public void printStatistics(PrintWriter printWriter) {
    printWriter.println("ConnectionPool : to-date-statistics");
    printWriter.println("-----------------------------------");
    printWriter.println("is open             :" + isOpen_);
    printWriter.println("pool url            :" + url_);
    printWriter.println("Initial size        :" + initialCount_);
    printWriter.println("Connections in pool :" + count_);
    printWriter.println("Connections outward :" + instancesOutwardCount_);
    printWriter.println("Callers waiting     :" + waitCount_);
    printWriter.println("Loops done so far   :" + loopCount_);
    printWriter.println("max. of con. out    :" + instancesOutwardMax_);
    printWriter.println("is in debug mode    :" + isDebug());
}



/**
 * Sets a new debug mode.
 * @author Michael Winkler, doubleSlash GmbH
 * @param newDebug boolean
 * @see isDebug() 
 */
public void setDebug(boolean newDebug) {
    debug_ = newDebug;
    if (newDebug)
        printStatistics(System.out);
}



/**
 * Takes an instance back into a pool. CAUTION! Do NOT call manually!
 * This method EXCLUSIVELY may be called from <code>PooledConnection</code>.
 * To repool a connection, use <code>PooledConnection.repool()</code> or <code>PooledConnection.close()</code>.
 * Creation date: (28.06.2000 18:36:31)
 * @param: <|>
 * @return:
 * @author: M.Winkler, doubleSlash GmbH
 *
 * @param pooledConnection org.jutils.jdbc.PooledConnection
 */
protected void takeInstance(PooledConnection pooledConnection) {
    if (pooledConnection != null) {
        if (instancesOutwardCount_ > 0)
            instancesOutwardCount_--;
        else {
            String message = "Did receive an instance that shouldnt be out.";
            throw new IllegalStateException(message);
        }
    }
    boolean isGood = false;
    try {
        if (isDebug()) {
            System.out.println("ConnectionPool.takeInstance(PooledConnection) : Committing incoming ...");
        }
        pooledConnection.commit();
        isGood = true;
        if (isDebug()) {
            System.out.println("ConnectionPool.takeInstance(PooledConnection) : Commit done.");
        }
    }
    catch (SQLException sqle) {
        System.err.println("ConnectionPool.takeInstance(PooledConnection) : Cannot commit incoming PooledConnection : : " + sqle.toString());
        sqle.printStackTrace(System.err);
        try {
            pooledConnection.rollback();
            isGood = true;
            if (isDebug()) {
                System.out.println("ConnectionPool.takeInstance(PooledConnection) : rolled back.");
            }
        }
        catch (SQLException sqle2) {
            System.err.println(" ConnectionPool.takeInstance(PooledConnection) : Cannot even rollback incoming PooledConnection : " + sqle2.toString());
            sqle2.printStackTrace(System.err);
        }
    }


    // is pool opened?
    if (isOpen_) {

        // is the passed instance okay?
        if (!isGood) {
            if (isDebug()) {
                System.out.println("ConnectionPool.takeInstance(PooledConnection) : replacing malicious incoming PooledConnection...");
            }
            pooledConnection.destroy();
            try {
                pooledConnection = new PooledConnection(pGetNativeConnection(), this);
                if (isDebug()) {
                    System.out.println("ConnectionPool.takeInstance(PooledConnection) : malicious PooledConnection replaced.");
                }
            }
            catch (SQLException sqle) {
                System.err.println("ConnectionPool.takeInstance(PooledConnection) : Cannot get a new connection : " + sqle.toString());
                sqle.printStackTrace(System.err);
                System.err.println("ConnectionPool.takeInstance(PooledConnection) : As a consequence, pool size lowered by 1.");
            }
        }

        // if open, add instance to the pool
        if (pooledConnection != null) {
            pooledConnection.lock();
            pool_[count_] = pooledConnection;
            count_++;
            if (isDebug()) {
                System.out.println("ConnectionPool.takeInstance(PooledConnection) : PooledConnection recycled.");
            }
        }
        else {
            // we are going down! destroy incoming connection.
            pooledConnection.destroy();
            if (isDebug()) {
                System.out.println("ConnectionPool.takeInstance(PooledConnection) : PooledConnection destroyed. ");
            }
            if (instancesOutwardCount_ < 1) {
                if (isDebug()) {
                    System.out.println("ConnectionPool.takeInstance(PooledConnection) : taken last instance out. destroying .... ");
                }
                this.destroy();
            }
        }
    }
    if (isDebug()) {
        printStatistics(System.out);
    }
}
}

