package org.jutils.jdbc;

/**
 * <p>------------------------------------------------------------------------------<br>
 * Name:     	SqlFactory<br>
 * Project:   	jutils<br>
 * Copyright: 	jutils.org, see license file<br>
 * Comment:		Provides an easy-to-use- factory for creating product specific sql copmmands.<br>
 * Creation date: (07.07.2000 17:44:33)   <br>
 * Version:   <pre>$Id: SqlFactory.java,v 1.22 2000/10/18 07:57:02 kkrafft2 Exp $</pre> <br>
 * ------------------------------------------------------------------------------<p>
 * @author: M.Winkler, doubleSlash GmbH
 */
import org.jutils.*;
import java.sql.*;
import java.text.*;
public interface SqlFactory {
    public static final String PRIMARY_KEY = "PRIMARY";
    public static final String LONG = "LONG";
    public static final String DATE = "DATE";
    public static final String DOUBLE = "DOUBLE";
    public static final String FLOAT = "FLOAT";
    public static final String BOOLEAN = "BOOLEAN";
    public static final String SHORT_STRING = "SHORT_STRING";
    public static final String STRING = "STRING";
    public static final String MEMO = "MEMO";
    public static final String BLOB = "BLOB";
    public static final String AS_IS = "AS_IS";



    /**
       * Used to compose an <code>alter table</code>-statement to create a single column.
       * <br>Creation date: (24.07.2000 09:45:43)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return StringBuffer the destination buffer.
       * @param tableName java.lang.String The name of the table where a column should be added.
       * @param columnsName java.lang.String   The column that shopuld be added.
       * @param fieldType java.lang.String The field type as defined as constant in <code>SqlFactory</code>.
       * @param defaultValue java.lang.Object The default value may be null.
       * @param StringBuffer The destination to append the statement. If null, then a new StringBuffer will be created.
       */
    StringBuffer composeAddColumn(String tableName, String columnsName, String fieldType, Object defaultValue, StringBuffer dest) throws ParseException;



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.sql.Blob
     * @param data byte[]
     */
    Blob composeBlob(Connection c, byte[] data) throws SQLException;



    /**
     * Composes a create Sequence sql statement.
     * <br>Creation date: (27.07.2000 17:18:05)<br>
     * @author: M.Winkler, doubleSlash GmbH
     *
     * @return java.lang.StringBuffer
     * @param name java.lang.String
     * @param initialValue long
     * @param increment long
     * @param dest java.lang.StringBuffer
     */
    StringBuffer composeCreateSequence(String name, long initialValue, long increment, StringBuffer dest);



    /**
       * Creates a "CREATE TABLE"-Statement.
       * <br>Creation date: (14.07.2000 17:36:13)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param tableName java.lang.String the name of the table to be created.
       * @param fields java.lang.String [] The names of the columns to be created.
       * @param types java.lang.String [] The types of the columns, in the same order like <code>columns</code>.
       * @param options java.lang.String [] The default values for the columns. May be null or partially null.
       */
    StringBuffer composeCreateTable(String tableName, String[] fields, String[] types, String[] defaultValues, StringBuffer destination) throws ParseException;



    /**
       * <br>Creation date: (19.07.2000 19:38:34)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.StringBuffer
       * @param sequenceName java.lang.String
       */
    StringBuffer composeDropSequence(String sequenceName, StringBuffer dest);



    /**
       * <br>Creation date: (19.07.2000 19:38:34)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.StringBuffer
       * @param tableName java.lang.String
       */
    StringBuffer composeDropTable(String tableName, StringBuffer dest);



    /**
       * Creates a "INSERT INTO"-Statement.
       * <br>Creation date: (14.07.2000 17:36:13)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param tableName java.lang.String the name of the table to insert into.
       * @param fields java.lang.String [] The names of the fields to insert.
       * @param types java.lang.String [] The types of the fields to insert (as defined via the constant arrays).
       * @param values java.lang.Object [] The values to be written into the db.

       */
    StringBuffer composeInsert(String tableName, String[] fields, String[] types, Object[] values, StringBuffer destination);



    /**
       * Composes a SQL Select statement, according to the provided values.
       * Creation date: (07.07.2000 17:49:46)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @param table java.lang.String [] The name of the tables to be accessed.
       * @param columns java.lang.String[] The names of the columns to be accessed.
       * @param expressions java.lang.String[]
       * @param modifiers java.lang.String [] a modifier (one of the constants).
       */
    StringBuffer composeQuery(String[] tables, String[] columns, String[] expressions, StringBuffer dest);



    /**
       * Creates a default update command containing the passed columns and new values.
       * <br>Creation date: (18.07.2000 11:28:01)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.StringBuffer
       * @param tableName java.lang.String
       * @param fields java.lang.String[] The field identifiers
       * @param newValues java.lang.Object[] The types to be used to convert from java value to db value.
       * @param dest java.lang.StringBuffer
       */
    StringBuffer composeUpdate(String tableName, String[] fields, String[] types, Object[] newValues, StringBuffer dest);



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return byte[]
     * @param fieldIdentifier java.lang.String
     * @param resultSet java.sql.ResultSet
     * @exception java.sql.SQLException The exception description.
     */
    byte[] loadBytes(String fieldIdentifier, ResultSet resultSet) throws java.sql.SQLException;



    /**
       * Parsed the string that is returned by the database into a boolean value.
       * This is do0ne to overcome proprietary data handling from different databases.
       * For example, on an Oracle database, there is no boolean value; its simulated
       * through a NUMBER type. If the callert of one of the composeXxx- methods uses
       * a composed statement to access a table, this method can be used to convert
       * the proprietary representation of then boolean type into the java type <code>boolean</code>.
       * <br>Creation date: (21.07.2000 14:55:06)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return boolean returns true or false, corresponding to the passed value.
       * @param dbReturnedStringValue java.lang.String A value that has been returned from the database to be a boolean value.
       * @exception java.text.ParseException Is thrown when the passed value cannot be interpreted as a boolean.
       */
    boolean toBooleanField(String dbReturnedStringValue) throws ParseException;



    /**
     * Converts the passed db entry to a java <code>Date</code> value.
     * <br>Creation date: (24.07.2000 10:04:53)<br>
     * @author: M.Winkler, doubleSlash GmbH
     * @return java.util.Date
     * @param fieldValue java.lang.String
     * @exception ParseException Thrown when the passed String is malformed.
     */
    java.util.Date toDateField(String fieldValue) throws ParseException;



    /**
       * Converts this string representation of a java value into the database proprietary representation.
       * <br>Creation date: (24.07.2000 09:58:58)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String The Database representation (as used in the composed sql statements) of the value.
       * @param value java.lang.OPbject The java value. Primitives must be passed in their shells.
       * @param fieldType java.lang.String The java field type identifier as defined in SqlFactory.
       */
    String toDbValue(Object value, String fieldType) throws ParseException;



    /**
       * Converts this string to the field representation, i.e. replaces ' through '' et. al.
       * <br>Creation date: (24.07.2000 10:03:14)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param value java.lang.String
       */
    String toDbValue(String value);



    /**
     * Converts the passed db value to the corresponding Jdbc Date type.
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.lang.String
     * @param date java.util.Date
     */
    String toDbValue(java.util.Date date);



    /**
       * <br>Creation date: (21.07.2000 15:00:36)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param param boolean A boolean that should be converted to a database specific field string.
       */
    String toDbValue(boolean param);



    /**
       * Converts the passed db entry to a java <code>double</code> value.
       * <br>Creation date: (24.07.2000 10:04:53)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param fieldValue java.lang.String
       */
    double toDoubleField(String fieldValue);



    /**
       * Converts the passed db entry to a java <code>float</code> value.
       * <br>Creation date: (11.10.2000 10:04:53)<br>
       *
       * @return java.lang.Float
       * @param fieldValue java.lang.String
       */
    float toFloatField(String fieldValue);



    /**
       * Converts the passed db entry to a java <code>long</code> value.
       * <br>Creation date: (24.07.2000 10:04:53)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param fieldValue java.lang.String
       */
    long toLongField(String fieldValue);



    /**
       * Removes all obsolete ' from the string.
       * <br>Creation date: (24.07.2000 10:04:53)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param fieldValue java.lang.String
       */
    String toStringField(String fieldValue);
}
