package org.jutils.text;
import java.util.*;

//------------------------------------------------------------------------------
//Name:      StringColumnizer
//Project:   jutils.org
//Copyright: jutils.org, see jutils-LICENSE file
//Comment:   Works like a combination of StringTokenizer and Vector.
//Version:   <pre>$Id: StringColumnizer.java,v 1.5 2001/01/09 14:58:56 winkler Exp $</pre>
//------------------------------------------------------------------------------

/**
 * Works like a combination of StringTokenizer and Vector. You can give in a string
 * and a separator, and access the separate tokens via a getter.
 *<br>Creation date: (31.08.2000 20:11:08)<br>
 * @author M.Winkler, doubleslash GmbH
 */
public class StringColumnizer {

   /** comment */
   public static final String DEFAULT_DELIMETER = " ";
   private StringList stringList_ = null;
   private int pos_ = 0;



   /**
   * Creates an empty StringColumnizer. For reuse only.
   */
   public StringColumnizer() {
     super();
     stringList_ = new StringList(25);
   }



   /**
   * Tokenizes the passed string with a default delimeter.
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @param string java.lang.String
   * @param separator java.lang.String
   */
   public StringColumnizer(String string) {
     this(string, DEFAULT_DELIMETER);
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @param string java.lang.String
   * @param separator java.lang.String
   */
   public StringColumnizer(String string, String separator) {
     // a bad guess is better than no guess, jye know...
     int approximateInitialSize = (int) Math.max(string.length() / 5, 10);
     stringList_ = new StringList(approximateInitialSize);
     startOver(string, separator);
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return int
   */
   public int count() {
     return stringList_.count();
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.lang.String
   */
   public String firstToken() {
     return stringList_.getStringAt(0);
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.lang.String
   * @param index int
   */
   public String getTokenAt(int index) {
     return stringList_.getStringAt(index);
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return boolean
   */
   public boolean hasMoreTokens() {
     return pos_ < stringList_.count();
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.lang.String
   */
   public String lastToken() {
     return stringList_.getLast();
   }



   /**
   * Starts the application.
   * @param args an array of command-line arguments
   */
   public static void main(java.lang.String[] args) {
     System.out.println("Testing StringColumnizer");
     String sampleString = "This is something to tokenize.";
     StringColumnizer columnizer = new StringColumnizer(sampleString);
     String[] returnedArray = new String[columnizer.count()];
     int index = -1;
     while (columnizer.hasMoreTokens()) {
       index++;
       returnedArray[index] = columnizer.nextToken();
       System.out.println(returnedArray[index]);
     }
     int errorCount = 0;

     // Testing firstToken und lastToken
     System.out.println("Testing firstToken und lastToken ...");
     String firstToken = columnizer.firstToken();
     String lastToken = columnizer.lastToken();
     if (!firstToken.equals(returnedArray[0])) {
       System.err.println("First token does not match : '" + firstToken + "' should be '" + returnedArray[0] + "'");
       errorCount++;
     }
     if (!lastToken.equals(returnedArray[returnedArray.length - 1])) {
       System.err.println("Last token does not match : '" + lastToken + "' should be '" + returnedArray[returnedArray.length - 1] + "'");
       errorCount++;
     }




     // Testing getTokenAt(int)
     System.out.println("Testing getTokenAt(int) ...");
     for (index = 0; index < returnedArray.length; index++) {
       if (!returnedArray[index].equals(columnizer.getTokenAt(index))) {
         System.err.println("Strings do not match at position " + index + " : '" + columnizer.getTokenAt(index) + "' should be '" + returnedArray[index] + "'");
         errorCount++;
       }
     }

     // Testing restart
     System.out.println("Testing restart ...");
     columnizer.restart();
     for (index = 0; index < returnedArray.length; index++) {
       if (!returnedArray[index].equals(columnizer.getTokenAt(index))) {
         System.err.println("Strings do not match at position " + index + " : '" + columnizer.getTokenAt(index) + "' should be '" + returnedArray[index] + "'");
         errorCount++;
       }
     }
     System.out.println(String.valueOf(errorCount) + " errors encountered.");
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.lang.String
   */
   public String nextToken() {
     String returnValue = stringList_.getStringAt(pos_);
     pos_++;
     return returnValue;
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   */
   public void restart() {
     pos_ = 0;
   }



   /**
      * Starts over with a new String to tokenize, and a new delimeter.
      * @author Michael Winkler, doubleSlash GmbH
      *
      * @param newString java.lang.String
      * @param newDelimeter java.lang.String
      */
   public void startOver(String newString, String newDelimeter) {
     stringList_.clear();
     StringTokenizer tokenizer = new StringTokenizer(newString, newDelimeter);
     pos_ = 0;
     while (tokenizer.hasMoreTokens())
       stringList_.add(tokenizer.nextToken());
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.lang.String[]
   * @param dest java.lang.String[]
   */
   public String[] toArray(String[] dest) {
     return stringList_.toArray(dest);
   }



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return org.jutils.text.StringList
   */
   public StringList toStringList() {
     String[] values = stringList_.toArray(null);
     return new StringList(values);
   }





/**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.util.List
   */
public List toList() {
   List returnValue = null;
   if ((stringList_ != null) && (stringList_.count() > 0)) {
      int listCount = stringList_.count();
      returnValue = org.jutils.runtime.Recycler.getVector();
      ((Vector) returnValue).ensureCapacity(listCount);
      for (int i = 0; i < listCount; i++) {
         returnValue.add(stringList_.getStringAt(i));
      }
   } else
      returnValue = org.jutils.collection.EmptyList.instance();
   return returnValue;
}
}
